package com.alpha.owner.ece4760;

import android.app.Activity;
import android.os.Bundle;
import android.os.Looper;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;
import android.location.Location;
import android.graphics.Color;

import com.google.android.gms.location.LocationServices;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationResult;

import java.text.DateFormat;
import java.util.Date;
import java.util.Locale;

public class MainActivity extends Activity implements ConnectResponse {

    /**
     * The desired interval for location updates. Inexact. Updates may be more or less frequent.
     */
    private static final long UPDATE_INTERVAL_IN_MILLISECONDS = 3000;

    /**
     * The fastest rate for active location updates. Exact. Updates will never be more frequent
     * than this value.
     */
    private static final long FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS =
            UPDATE_INTERVAL_IN_MILLISECONDS / 2;

    /**
     * Provides access to the Fused Location Provider API.
     */
    private FusedLocationProviderClient mFusedLocationClient;

    /**
     * Stores parameters for requests to the FusedLocationProviderApi.
     */
    private LocationRequest mLocationRequest;

    /**
     * Callback for Location events.
     */
    private LocationCallback mLocationCallback;

    /**
     * Represents a geographical location.
     */
    private Location mCurrentLocation;

    /**
     * Time when the location was updated represented as a String.
     */
    private String mLastUpdateTime;

    /**
     * Client connection to TCP server.
     */
    private TCPClient mTCPClient;

    // UI Widgets
    private EditText mAddressEditText;
    private EditText mPortEditText;
    private EditText mMessageEditText;
    private Button mConnectButton;
    private Button mSendButton;
    private Button mTakeOffButton;
    private Button mLandButton;
    private Button mStopButton;

    // Text
    private TextView mConnectionTextView;
    private TextView mLastUpdateTimeTextView;
    private TextView mLatitudeTextView;
    private TextView mLongitudeTextView;

    // Labels.
    private String mLatitudeLabel;
    private String mLongitudeLabel;
    private String mLastUpdateTimeLabel;
    private String mTakeOffLabel = "$CONTROL$ Take Off";
    private String mLandLabel = "$CONTROL$ Land";
    private String mStopLabel = "$CONTROL$ Stop";
    private String mConnectedLabel = "Connected";
    private String mNotConnectedLabel = "No Connection";

    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        this.mAddressEditText = (EditText) findViewById(R.id.address_edit_text);
        this.mPortEditText = (EditText) findViewById(R.id.port_edit_text);
        this.mMessageEditText = (EditText) findViewById(R.id.message_edit_text);

        this.mConnectButton = (Button) findViewById(R.id.connect_button);
        this.mSendButton = (Button)findViewById(R.id.send_button);
        this.mTakeOffButton = (Button)findViewById(R.id.take_off_button);
        this.mLandButton = (Button)findViewById(R.id.land_button);
        this.mStopButton = (Button)findViewById(R.id.stop_button);

        this.mConnectionTextView = (TextView) findViewById(R.id.connection_text);
        this.mLatitudeTextView = (TextView) findViewById(R.id.latitude_text);
        this.mLongitudeTextView = (TextView) findViewById(R.id.longitude_text);
        this.mLastUpdateTimeTextView = (TextView) findViewById(R.id.last_update_time_text);

        // Set labels.
        this.mConnectedLabel = getResources().getString(R.string.connected_label);
        this.mNotConnectedLabel = getResources().getString(R.string.not_connected_label);
        this.mLatitudeLabel = getResources().getString(R.string.latitude_label);
        this.mLongitudeLabel = getResources().getString(R.string.longitude_label);
        this.mLastUpdateTimeLabel = getResources().getString(R.string.last_update_time_label);

        this.mTCPClient = new TCPClient();

        this.mConnectButton.setOnClickListener(new ConnectListener(this.mAddressEditText, this.mPortEditText, this.mTCPClient, this));
        this.mSendButton.setOnClickListener(new SendListener(this.mMessageEditText, this.mTCPClient));
        this.mTakeOffButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //sends the message to the server
                new SendTask(mTCPClient).execute(mTakeOffLabel);
            }
        });
        this.mLandButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //sends the message to the server
                new SendTask(mTCPClient).execute(mLandLabel);
            }
        });
        this.mStopButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //sends the message to the server
                new SendTask(mTCPClient).execute(mStopLabel);
            }
        });

        new ConnectTask(this.mTCPClient, this).execute();

        this.mFusedLocationClient = LocationServices.getFusedLocationProviderClient(this);

        mLastUpdateTime = "";

        mFusedLocationClient = LocationServices.getFusedLocationProviderClient(this);

        // Kick off the process of building the LocationCallback, LocationRequest, and
        // LocationSettingsRequest objects.
        //createLocationCallback();
        //createLocationRequest();

        if (ActivityCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            // TODO: Consider calling
            //    ActivityCompat#requestPermissions
            // here to request the missing permissions, and then overriding
            //   public void onRequestPermissionsResult(int requestCode, String[] permissions,
            //                                          int[] grantResults)
            // to handle the case where the user grants the permission. See the documentation
            // for ActivityCompat#requestPermissions for more details.
            Toast.makeText(this, "First enable LOCATION ACCESS in settings.", Toast.LENGTH_LONG).show();
            return;
        }

        //noinspection MissingPermission
        //mFusedLocationClient.requestLocationUpdates(mLocationRequest,
                //mLocationCallback, Looper.myLooper());
    }

    @Override
    public void processFinish(boolean output) {
        if (output) {
            this.mConnectionTextView.setText(this.mConnectedLabel);
            this.mConnectionTextView.setTextColor(Color.rgb(105, 229, 103));
        }
        else {
            this.mConnectionTextView.setText(this.mNotConnectedLabel);
            this.mConnectionTextView.setTextColor(Color.rgb(229, 75, 75));
        }
    }

    /**
     * Sets up the location request. Android has two location request settings:
     * {@code ACCESS_COARSE_LOCATION} and {@code ACCESS_FINE_LOCATION}. These settings control
     * the accuracy of the current location. This sample uses ACCESS_FINE_LOCATION, as defined in
     * the AndroidManifest.xml.
     * <p/>
     * When the ACCESS_FINE_LOCATION setting is specified, combined with a fast update
     * interval (5 seconds), the Fused Location Provider API returns location updates that are
     * accurate to within a few feet.
     * <p/>
     * These settings are appropriate for mapping applications that show real-time location
     * updates.
     */
    private void createLocationRequest() {
        mLocationRequest = new LocationRequest();

        // Sets the desired interval for active location updates. This interval is
        // inexact. You may not receive updates at all if no location sources are available, or
        // you may receive them slower than requested. You may also receive updates faster than
        // requested if other applications are requesting location at a faster interval.
        mLocationRequest.setInterval(UPDATE_INTERVAL_IN_MILLISECONDS);

        // Sets the fastest rate for active location updates. This interval is exact, and your
        // application will never receive updates faster than this value.
        mLocationRequest.setFastestInterval(FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS);

        mLocationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
    }

    /**
     * Creates a callback for receiving location events.
     */
    private void createLocationCallback() {
        mLocationCallback = new LocationCallback() {
            @Override
            public void onLocationResult(LocationResult locationResult) {
                super.onLocationResult(locationResult);

                mCurrentLocation = locationResult.getLastLocation();
                mLastUpdateTime = DateFormat.getTimeInstance().format(new Date());
                String payload = String.format(Locale.US, "$GPS,%f,%f,%f", mCurrentLocation.getLatitude(), mCurrentLocation.getLongitude(), mCurrentLocation.getAltitude());
                mCurrentLocation.getLatitude();
                new SendTask(mTCPClient).execute(payload);
                updateLocationUI();
            }
        };
    }

    /**
     * Sets the value of the UI fields for the location latitude, longitude and last update time.
     */
    private void updateLocationUI() {
        if (mCurrentLocation != null) {
            mLatitudeTextView.setText(String.format(Locale.ENGLISH, "%s: %f", mLatitudeLabel,
                    mCurrentLocation.getLatitude()));
            mLongitudeTextView.setText(String.format(Locale.ENGLISH, "%s: %f", mLongitudeLabel,
                    mCurrentLocation.getLongitude()));
            mLastUpdateTimeTextView.setText(String.format(Locale.ENGLISH, "%s: %s",
                    mLastUpdateTimeLabel, mLastUpdateTime));
        }
    }
}
