package com.alpha.owner.ece4760;

import android.util.Log;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.Socket;

public class TCPClient {

    private String SERVERIP = "192.168.4.1"; //your computer IP address
    private int SERVERPORT = 5000;
    private Socket socket = null;
    private OnMessageReceived messageListener = null;
    private boolean run = false;

    PrintWriter out;
    BufferedReader in;

    public TCPClient() {
    }

    /**
     *  Constructor of the class. OnMessagedReceived listens for the messages received from server
     */
    public TCPClient(OnMessageReceived listener) {
        this.messageListener = listener;
    }

    public void setAddress(String address) {
        this.SERVERIP = address;
    }

    public void setPort(int port) {
        this.SERVERPORT = port;
    }

    /**
     * Sends the message entered by client to the server
     * @param message text entered by client
     */
    public void sendMessage(String message){
        if (out != null && !out.checkError()) {
            out.println(message);
            out.flush();
        }
    }

    public void stopClient() {
        this.run = false;
        try {
            this.socket.close();
            this.socket = null;
        } catch (Exception e) {
            Log.e("TCP", "S: Error", e);
        }
    }

    public boolean connect() {

        run = true;

        if (this.socket != null) {
            try {
                this.socket.close();
                this.socket = null;
            } catch (Exception e) {
                Log.e("TCP", "S: Error", e);
            }
        }

        try {
            //here you must put your computer's IP address.
            InetAddress serverAddr = InetAddress.getByName(SERVERIP);

            Log.e("TCP Client", "C: Connecting...");

            //create a socket to make the connection with the server
            this.socket = new Socket(serverAddr, SERVERPORT);

            try {

                //send the message to the server
                out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);

                Log.e("TCP Client", "C: Sent.");

                Log.e("TCP Client", "C: Done.");

                //receive the message which the server sends back
                in = new BufferedReader(new InputStreamReader(socket.getInputStream()));

            } catch (Exception e) {

                Log.e("TCP", "S: Error", e);

                return false;

            }

        } catch (Exception e) {

            Log.e("TCP", "C: Error", e);

            return false;

        }

        return true;

    }

    //Declare the interface. The method messageReceived(String message) will must be implemented in the MainActivity
    //class at on asynckTask doInBackground
    public interface OnMessageReceived {
        public void messageReceived(String message);
    }
}
